package dev.pfaff.altargui.pathfinder;

import java.lang.foreign.Arena;
import java.lang.foreign.MemorySegment;
import java.lang.foreign.ValueLayout;

import static dev.pfaff.altargui.pathfinder.Pathfinder.*;
import static java.lang.foreign.ValueLayout.JAVA_BYTE;

public record PFSceneProxyRef(MemorySegment ptr) {
	public static PFSceneProxyRef createWithSequentialExecutor(PFRendererMode mode) {
		return new PFSceneProxyRef(constructNativeObject(ML_PFSceneProxy,
				ptr -> {
					try (var arena = Arena.ofConfined()) {
						var modePtr = allocRendererMode(mode, arena);
						return (int) MH_PFSceneProxyInitWithSequentialExecutor.invokeExact(ptr, modePtr);
					}
				},
				ptr -> MH_PFSceneProxyDeinit.invokeExact(ptr)));
	}

	private static MemorySegment allocRendererMode(PFRendererMode mode, Arena arena) {
		return arena.allocateFrom(JAVA_BYTE, mode.value).asSlice(0, ML_PFRendererMode);
	}

	public static PFSceneProxyRef createWithRayonExecutor(PFRendererMode mode) {
		return new PFSceneProxyRef(constructNativeObject(ML_PFSceneProxy,
				ptr -> {
					try (var arena = Arena.ofConfined()) {
						var modePtr = allocRendererMode(mode, arena);
						return (int) MH_PFSceneProxyInitWithRayonExecutor.invokeExact(ptr, modePtr);
					}
				},
				ptr -> MH_PFSceneProxyDeinit.invokeExact(ptr)));
	}

	/**
	 * The {@code scene} will be moved out of the reference by this call.
	 */
	public void replaceScene(PFSceneRef scene) {
		try {
			scene.setManaged(false);
			MH_PFSceneProxyReplaceScene.invokeExact(this.ptr, scene.ptr());
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	public void buildAndRenderGL(PFGLRendererRef renderer, PFBuildOptionsRef options) {
		try {
			MH_PFSceneProxyBuildAndRenderGL.invokeExact(this.ptr(), renderer.ptr(), options.ptr());
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}
}
